/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Processador de Comandos - Descrio na prpria classe.
 ************************************************************************/

using System.Collections;

namespace ERWon
{
	/// <summary>
	/// O processador de comandos coordena a funcionalidade de desfazer e refazer.
	/// Armazena pilhas de Commands, uma classe base cujas classes derivadas so os
	/// comandos efetivos que so feitos e desfeitos.
	/// </summary>
	public class CommandProcessor
	{
		Stack m_UndoStack;
		Stack m_RedoStack;

		/// <summary>Esvazia uma pilha de comandos</summary>
		protected void EmptyStack (Stack Commands)
		{
			Commands.Clear ();
		}


		/// <summary>
		/// Percorre uma pilha de comandos e apaga os comandos referentes
		/// a determinado documento.
		/// </summary>
		protected void FilterStack (Stack Commands, Document Filter)
		{
			ArrayList Temp = new ArrayList ();

			foreach (Command EachCommand in Commands)
			{
				if (EachCommand.ParentDocument != Filter)
					Temp.Add (EachCommand);
			}

			EmptyStack (Commands);
			Temp.Reverse ();

			foreach (Command EachCommand in Temp)
			{
				Commands.Push (EachCommand);
			}
		}

		public CommandProcessor()
		{			
			m_UndoStack = new Stack ();
			m_RedoStack = new Stack ();
		}

		/// <summary>Executa o comando passado e empilha na pilha de desfazer.</summary>
		public void Execute (Command Cmd)
		{
			Cmd.Execute ();

			EmptyStack (m_RedoStack);

			if (Cmd.CanUndo)
				m_UndoStack.Push (Cmd);
			else
				EmptyStack (m_UndoStack);
		}


		/// <summary>
		/// Desfaz o ultimo comando (topo da pilha de desfazer)
		/// e o move para a pilha de refazer
		/// </summary>
		public void Undo ()
		{
			if (m_UndoStack.Count < 1)
				return;

			Command Undo = m_UndoStack.Pop () as Command;
			if (Undo != null)
			{
				Undo.Undo ();
				m_RedoStack.Push (Undo);
			}
		}


		/// <summary>
		/// Refaz o ultimo comando desfeito (topo da pilha de refazer)
		/// e o move para pilha do desfazer
		/// </summary>
		public void Redo ()
		{
			if (m_RedoStack.Count < 1)
				return;

			Command Redo = m_RedoStack.Pop () as Command;
			if (Redo != null)
			{
				Redo.Execute ();
				m_UndoStack.Push (Redo);
			}
		}


		/// <summary>Determina se o comando UNDO est disponvel.</summary>
		public bool CanUndo ()
		{
			return m_UndoStack.Count > 0;
		}

		/// <summary>Determina se o comando REDO est disponvel.</summary>
		public bool CanRedo ()
		{
			return m_RedoStack.Count > 0;
		}


		/// <summary>Retorna o texto para o menu UNDO, baseado no nome do comando.</summary>
		public string UndoText ()
		{
			if (m_UndoStack.Count < 1)
				return "";

			Command Next = m_UndoStack.Peek () as Command;
			if (Next == null)
				return null;

			return Next.ShortDescription;
		}

		/// <summary>Retorna o texto para o menu REDO, baseado no nome do comando.</summary>
		public string RedoText ()
		{
			if (m_RedoStack.Count < 1)
				return "";

			Command Next = m_RedoStack.Peek () as Command;
			if (Next == null)
				return null;

			return Next.ShortDescription;
		}

		/// <summary>
		/// Percorre as pilha de comandos e apaga os comandos referentes
		/// ao documento fechado.
		/// </summary>
		public void DocumentClosed (Document Closed)
		{
			FilterStack (m_UndoStack, Closed);
			FilterStack (m_RedoStack, Closed);
		}
	}

}
