/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe abstrata de Elementos. Todos elementos no documento derivam
 * desta classe
 ************************************************************************/

using System;
using System.Runtime.Serialization;
using System.ComponentModel;
using System.Drawing;

namespace ERWon
{
	/// <summary>Todos elementos pertencentes ao documento derivam desta classe</summary>
	[Serializable]
	public abstract class DocElement : ISerializable
	{
		/// <summary>Indica se o objeto est ou no selecionado</summary>
		protected bool m_iSelected = false;
		[Browsable (false)]
		public bool Selected { get { return m_iSelected; } }

		/// <summary>Elemento pai</summary>
		/// A idia  que possam haver elementos dentro de outros.
		/// Para documentos isto deve ser nulo.
		protected DocElement m_Parent;
		[Browsable (false)] public DocElement Parent { get { return m_Parent; } set { m_Parent = value; } }
		
		/// <summary>Atualmente todos elementos so filhos diretos do Document.
		/// Logo, o m_Parent pode ser casteado para IDocument.
		/// </summary>
		[Browsable (false)]
		public IDocument Document { get { return m_Parent as IDocument; } set { m_Parent = value as DocElement; } }


		/// <summary>Retorna o documento pai do elemento</summary>
		public Document ParentDocument()
		{ 
			DocElement element = this;

			while (element.m_Parent != null)
			{
				element = element.m_Parent;
			}

			return element as Document;
		}


		/// <summary>
		/// Ao fazer uma cpia de um DocElement, esta propriedade
		/// aponta para o DocElement original. Serve para restaurar os
		/// relacionamentos entre os objetos copiados. Aps o trmino
		/// da cpia, deve ser limpado (null) para evitar problemas
		/// com serializao e coletor de lixo.
		/// </summary>
		[Browsable (false)]
		public DocElement OriginalElement
		{ 
			get { return m_OriginalElement; } 
			set { if (value == null) m_OriginalElement = null; else throw new System.Exception(); }
		}
		protected DocElement m_OriginalElement;



		/// <summary>Construtor</summary>
		public DocElement ()
		{
		}
		/// <summary>Constutor de serializao (ISerializable)</summary>
		/// Basicamente, s vem da deserializao o que estiver dentro deste construtor,
		/// e s entra na serializao o que estiver em GetObjectData
		protected DocElement (SerializationInfo Info, StreamingContext Context)
		{
			m_Parent = Info.GetValue ("Parent", typeof (object)) as DocElement;
		}

		/// <summary>Para serializao... ISerializable</summary>
		/// Basicamente, s vem da deserializao o que estiver dentro deste construtor,
		/// e s entra na serializao o que estiver em GetObjectData
		public virtual void GetObjectData (SerializationInfo Info, StreamingContext Context)
		{
			Info.AddValue ("Parent", m_Parent);
		}


	
		/// <summary>Seleciona o elemento</summary>
		/// O status (selecionado ou no) influi no comportamento do mouse e
		/// nos eventos de desenho (OnPaint).
		public void Select ()
		{
			m_iSelected=true;
		}

		/// <summary>De-seleciona o elemento</summary>
		public void Deselect ()
		{
			m_iSelected=false;
		}

		/// <summary>Determina se este elemento  filho do elemento Parent</summary>
		public bool ChildOf (DocElement Parent)
		{			
			for (DocElement element = this.Parent; element != null; element = element.Parent)
			{
				if (element == Parent)
					return true;
			}

			return false;
		}


		/// <summary>Cria uma nova instncia do DocElement (e derivados) e aponta para o DocElement original</summary>
		///  usado para operaes de clipboard
		public virtual DocElement CreateClipboardCopy ()
		{
			System.Type MyType = this.GetType ();
			System.Type[] NoParameters = new System.Type[0];
			System.Reflection.ConstructorInfo Constructor = MyType.GetConstructor (NoParameters);
			DocElement NewObject = Constructor.Invoke (NoParameters) as DocElement;
			NewObject.m_OriginalElement = this;
			return NewObject;
		}

		/// <summary>Arrasta o objeto por um dado deslocamento</summary>
		public virtual void Drag (int iDeltaX, int iDeltaY)
		{
		}

		/// <summary>
		/// Desenha um ponto especial nas coordenadas dadas.
		/// Usado nas classes derivadas.
		/// </summary>
		protected void DrawSpecialPoint (System.Drawing.Graphics g, int X, int Y)
		{
			const int Size = 5;
			Brush ConnectionBrush = new SolidBrush (Color.Black);
			g.FillRectangle (ConnectionBrush, X-(Size/2), Y-(Size/2), Size, Size);
		}

	}
}
