/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Aqui h Interfaces pblicas, e alguns tipos enumerados
 ************************************************************************/

using System.ComponentModel;
using System.Collections;

namespace ERWon
{
	/// <summary>Os documentos (diagramas ER) implementam esta interface</summary>
	public interface IDocument
	{
		/// <summary>Lista de Boxes no Documento</summary>
		/// Box  um elemento genrico, que pode ser redimensionado, movido, etc
		/// As entidades so derivadas dele
		/// Caixas de comentrios tambm.
		/// Talvez possa ser extendido para ser um editor completo de diagramas de vrios tipos.
		ArrayList Boxes { get; }

		/// <summary>Lista de Lines no Documento</summary>
		/// Line  um elemento genrico que vincula dois Boxes.
		/// Relacionamentos so derivados dele.
		/// Talvez possa ser extendido para ser um editor completo de diagramas de vrios tipos.
		ArrayList Lines { get; }

		/// <summary>Cor de background do documento</summary>
		System.Drawing.Color Background { get; }

		bool Selected { get; }

		/// <summary>O proprio documento (se selecionado) desenha-se (uma borda)</summary>
		void Draw (System.Drawing.Graphics g, Control c);

	}


	/// <summary>Todos elementos do documento implementam esta interface (ou derivada)
	/// Define funes para testar se elemento est sob o mouse, e funo de desenho.
	/// </summary>
	//[ Serializable() ]
	public interface IDocumentElement
	{		
		/// <summary>Documento pai</summary>
		[Browsable (false)]
		IDocument Document { get; set; }

		/// <summary>Cada objeto  responsvel pelo seu prprio desenho...</summary>
		void Draw (System.Drawing.Graphics g);

		/// <summary>Testa se o elemento est sob o mouse</summary>
		/// <returns>Returns true if the mouse hit the object, false otherwise</returns>
		bool Hit (int X, int Y);
	}

	
	/// <summary>Boxes devem implementar esta interface.</summary>
	//[ Serializable() ]
	public interface IBox : IDocumentElement 
	{
		/// <summary>Posio do elemento</summary>
		/// A posio de elementos Box  do tipo:
		/// X0,Y0 = canto superior esquerdo
		/// X1, Y1 = canto inferior direito
		/// Foi a melhor implementao que consegui para as operaes de redimensionamento
		/// e outras funcionalidades que implementei
		[Category ("Position"), Description ("X position of this Box")]
		int X0 { get; set; }
		int X1 { get; set; }
		[Category ("Position"), Description ("Y position of this Box")]
		int Y0 { get; set; }
		int Y1 { get; set; }

		/// <summary>
		/// Propriedades derivadas - usadas em geral para desenho do elemento
		/// ou para propertygrid
		/// </summary>
		[Category ("Dimensions"), Description ("Width of this Box")]
		int Width { get; set; }
		[Category ("Dimensions"), Description ("Height of this Box")]
		int Height { get; set; }
		[Browsable (false)]		int Left   { get; }
		[Browsable (false)]		int Right  { get; }
		[Browsable (false)]		int Top    { get; }
		[Browsable (false)]		int Bottom { get; }

		/// <summary>
		/// Testa se as coordenadas esto sobre algum ponto especial do elemento
		/// Pontos especiais incluem os extremos usados para redimensionar boxes
		/// </summary>
		uint HitSpecialPoint (int X, int Y);

		/// <summary>
		/// Testa se as coordenadas esto sobre algum extremo do box
		/// </summary>
		Box_Corner GetCorner (int X, int Y);

		/// <summary>Move o elemento da quantidade passada</summary>
		/// <param name="iDeltaX"></param>
		/// <param name="iDeltaY"></param>
		void Drag(int iDeltaX, int iDeltaY);

	}

	
	/// <summary>Lines devem implementar esta interface</summary>
	public interface ILine : IDocumentElement
	{
		/// <summary>Box de Origem</summary>
		[Browsable(false)]
		IBox Source { get; set; }

		/// <summary>Box de Destino</summary>
		[Browsable(false)]
		IBox Destination { get; set; }
	}	


	/// <summary>
	/// Corners (que so pontos especiais) de um Box - para arrastar e redimensionar
	/// </summary>
	public enum Box_Corner
	{
		TopLeft, Top, TopRight, Right, BottomRight, Bottom, BottomLeft, Left,
		None
	}


	/// <summary>
	/// Botes da barra de ferramentas...
	/// </summary>
	public enum botao_toolbar
	{
		None, SelectionTool, Add_Entity, Add_NonIdentifying_Relationship, Add_Identifying_Relationship, Add_Comment, Zoom_Inc, Zoom_Dec, Zoom_Clear
	}




	/// <summary>
	/// Interface de Callback com o MainForm.
	/// Usado pelo Control, pelo Document, e pelo MdiChild.
	/// </summary>
	public interface IControlCallbacks
	{
		void DocControl_BoxPicked (IBox Picked, bool fResetSelectionSet);
		void DocControl_LinePicked (ILine Picked, bool fResetSelectionSet);
		void DocControl_DocumentPicked (IDocument document, bool fResetSelectionSet);

		void DocControl_DragBegin (IDocument document, int iStartX, int iStartY);
		void DocControl_DragContinue (int iNewX, int iNewY);
		void DocControl_DragEnd (int iNewX, int iNewY);

		void DocControl_ResizeBegin (IDocument document, IBox Picked, Box_Corner BoxCorner, int iNewX, int iNewY);
		void DocControl_ResizeContinue (IBox Picked, int iNewX, int iNewY);
		void DocControl_ResizeEnd (int iNewX, int iNewY);

		void DocControl_CreateBox (IDocument document, IBox NewBox, int iNewX, int iNewY);
		void DocControl_CreateLine (IDocument document, ILine NewLine, int iNewX, int iNewY);

		/// <summary>Retorna a ferramenta selecionada na barra de ferramentas</summary>
		botao_toolbar BotaoSelecionado();

		/// <summary>Ativa o Selection Tool (cursor), de selecionar/mover objetos.</summary>
		void Toolbar_Reset();
	};


	/// <summary>
	/// Formulrios que mostram documentos devem implementar esta interface.
	/// No caso, apenas o MDI-Child.
	/// </summary>
	public interface IView
	{
		void Repaint ();
	}




}