/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe Box  uma classe abstrata que  herdada por qualquer
 * elemento que seja arrastvel e redimensionvel. Isso inclui as entidades
 * e comentrios.
 ************************************************************************/

using System;
using System.ComponentModel;
using System.Drawing;
using System.Runtime.Serialization;

namespace ERWon
{
	/// <summary>
	/// Classe Box  uma classe abstrata que  herdada por qualquer
	/// elemento que seja arrastvel e redimensionvel. Isso inclui as entidades
	/// e comentrios.
	/// </summary>
	[ DefaultProperty ("EntityName"), Serializable ]
	public abstract class Box : DocElement, IBox, ISerializable
	{

		public int m_MinHeight=40;
		public int m_MinWidth=40;

		/// <summary>Coordenadas do elemento</summary>
		protected int m_X0;
		protected int m_X1;
		protected int m_Y0;
		protected int m_Y1;

		/// <summary>Nome do Box</summary>
		protected string m_strText;

		/// <summary>Nmero de pontos especiais neste elemento</summary>
		/// Usado tanto para desenho quanto para HitTest
		[Browsable (false)]
		public abstract uint SpecialPoints {get;} 

		/// <summary>Coordenadas do elemento</summary>
		[Description ("X coordinate of the Box's position"), Category ("Position")]
		public int X0 { get { return m_X0; } set { m_X0 = value; } }
		[Browsable (false)]
		public int X1 { get { return m_X1; } set { m_X1 = value; } }
		[Description ("Y coordinate of the Box's position"), Category ("Position")]
		public int Y0 { get { return m_Y0; } set { m_Y0 = value; } }
		[Browsable (false)]
		public int Y1 { get { return m_Y1; } set { m_Y1 = value; } }


		/// <summary>Largura do elemento</summary>
		[Description ("Width of the Box"), Category ("Position")]
		public int Width { get { return m_X1 - m_X0; } set { m_X1 = m_X0 + value; } }

		/// <summary>Altura do elemento</summary>
		[Description ("Height of the Box"), Category ("Position")]
		public int Height { get { return m_Y1 - m_Y0; } set { m_Y1 = m_Y0 + value; } }

		/// <summary>Nome do elemento</summary>
		[Browsable (false)]
		[Description ("Text of the Box"), Category ("Appearance")]
		public virtual string Text { get { return m_strText; } set { m_strText = value; } }


		/// <summary>Propriedades (de tamanho) derivadas</summary>
		[Browsable (false)] public int Left   { get { return m_X0; } }
		[Browsable (false)]	public int Right  { get { return m_X1; } }
		[Browsable (false)]	public int Top    { get { return m_Y0; } }
		[Browsable (false)]	public int Bottom { get { return m_Y1; } }
		
		/// <summary>Fonte usada ao desenhar partes texto do elemento</summary>
		protected static Font ms_Font = new Font ("Arial", 10);
		Size m_TextSize = new Size ();

		/// <summary>Cor do elemento</summary>
		protected Color m_Foreground = System.Drawing.Color.White;
		public Color Foreground { get { return m_Foreground; } set { m_Foreground = value; } }

		/// <summary>Cor de fundo do elemento</summary>
		protected Color m_Background = System.Drawing.Color.Blue;
		public Color Background { get { return m_Background; } set { m_Background = value; } }



		/// <summary>Construtor</summary>
		public Box ()
		{
			Height = this.m_MinHeight;
			Width = this.m_MinWidth;
			m_strText = "";
		}

		/// <summary>Construtor de serializao</summary>
		protected Box (SerializationInfo Info, StreamingContext Context) : base (Info, Context)
		{
			m_X0 = Info.GetInt32 ("X0");
			m_Y0 = Info.GetInt32 ("Y0");
			m_X1 = Info.GetInt32 ("X1");
			m_Y1 = Info.GetInt32 ("Y1");
			m_strText = Info.GetString ("Text");
			
			// Foreground color
			//TODO - testar melhor isso, e tirar o try-catch
			try { m_Foreground = Util.StringToColor (Info.GetString ("ColorForeground")); } 
			catch { m_Foreground = System.Drawing.Color.White; }

			// Background color
			try { m_Background = Util.StringToColor (Info.GetString ("ColorBackground")); } 
			catch { m_Background = System.Drawing.Color.Blue; }
		}

		/// <summary>Deserializao</summary>
		public override void GetObjectData (SerializationInfo Info, StreamingContext Context)
		{
			base.GetObjectData (Info, Context);
			Info.AddValue ("X0", m_X0);
			Info.AddValue ("Y0", m_Y0);
			Info.AddValue ("X1", m_X1);
			Info.AddValue ("Y1", m_Y1);
			Info.AddValue ("Text", m_strText);
			Info.AddValue ("ColorForeground", Util.ColorToString (m_Foreground)); //, "" + m_Foreground.R + " " + m_Foreground.G + " " + m_Foreground.B);
			Info.AddValue ("ColorBackground", Util.ColorToString (m_Background)); // "" + m_Background.R + " " + m_Background.G + " " + m_Background.B);
		}


		/// <summary>A interface obriga a ter este mtodo, mas s as classes
		/// derivadas  que vo implement-lo na prtica...</summary>
		abstract public void Draw (Graphics g);

		/// <summary>Move o elemento por um dado deslocamento</summary>
		public override void Drag (int iDeltaX, int iDeltaY)
		{
			m_X0 += iDeltaX;
			m_X1 += iDeltaX;
			m_Y0 += iDeltaY;
			m_Y1 += iDeltaY;
		}

		/// <summary>
		/// Indica se as coordenadas dadas esto acima do elemento ou de
		/// algum de seus pontos especiais
		/// </summary>
		public virtual bool Hit (int X, int Y)
		{
			if (HitSpecialPoint (X, Y) > 0)
				return true;

			if ((X > Left) &&
				(X < Right) &&
				(Y < Bottom) &&
				(Y > Top))
				return true;

			return false;
		}

		/// <summary>
		/// Se as coordenadas dadas esto acima de algum ponto especial
		/// do elemento, retorna o ID do ponto. Caso contrrio, 0.
		/// </summary>
		public virtual uint HitSpecialPoint (int X, int Y) 
		{
			for (uint u = 1; u <= SpecialPoints; u++)
			{
				if (Util.HitTest (GetSpecialPoint (u), X, Y))
					return u;
			}
			return 0;
		}

		/// <summary>
		/// Os boxes tem 8 pontos especiais que so extremos usados para redimensionamento.
		/// Retorna se algum deles (e qual) est sob as coordenadas dadas.
		/// </summary>
		public virtual Box_Corner GetCorner (int X, int Y) 
		{
			uint tmp = HitSpecialPoint(X, Y);
			switch (tmp)
			{
				case 1:
					return Box_Corner.TopLeft;
				case 2:
					return Box_Corner.Top;
				case 3:
					return Box_Corner.TopRight;
				case 4:
					return Box_Corner.Right;
				case 5:
					return Box_Corner.BottomRight;
				case 6:
					return Box_Corner.Bottom;
				case 7:
					return Box_Corner.BottomLeft;
				case 8:
					return Box_Corner.Left;
				default:
					return Box_Corner.None;
			}
		}

		/// <summary>Retorna coordenadas dos pontos especiais</summary>
		/// Usado para desenha-los e para HitTest.
		public abstract Point GetSpecialPoint (uint ConnectionID);

		
		//TODO
		public void SizeToFitText (System.Drawing.Graphics g, string text)
		{
			Size XSize = g.MeasureString (text, ms_Font).ToSize ();
			m_TextSize = g.MeasureString (text, ms_Font).ToSize ();
			Width = m_TextSize.Width + XSize.Width;
			Height = m_TextSize.Height + XSize.Height;

			Width = Math.Max (Width, 25);
			Height = Math.Max (Height, 25);
		}


		// TODO
		public void DrawText (System.Drawing.Graphics g, Brush Color)
		{
			StringFormat Format = new StringFormat (StringFormatFlags.NoClip);
			g.DrawString (m_strText, ms_Font, Color, X0 - (m_TextSize.Width / 2), Y0 - (m_TextSize.Height / 2), Format);
		}

		/// <summary>Desenha os pontos especiais do elemento</summary>
		protected void DrawSpecialPoints (System.Drawing.Graphics g)
		{
			for (uint u = 1; u <= SpecialPoints; u++)
			{
				Point Connection = GetSpecialPoint (u);
				DrawSpecialPoint (g, Connection.X, Connection.Y);
			}
		}


		/// <summary>Cria cpia do elemento para usar em operaes de clipboard.</summary>
		public override DocElement CreateClipboardCopy ()
		{
			Box Copy = base.CreateClipboardCopy () as Box;

			Copy.m_X0 = this.m_X0;
			Copy.m_Y0 = this.m_Y0;
			Copy.m_X1 = this.m_X1;
			Copy.m_Y1 = this.m_Y1;
			Copy.m_strText = this.m_strText;
			Copy.Foreground = this.Foreground;
			Copy.Background = this.Background;

			return Copy as DocElement;
		}

	}
}