/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe Document
 * Representa um documento (ou seja, um diagrama ER)
 ************************************************************************/

using System;
using System.Collections;
using System.Drawing;
using System.ComponentModel;
using System.Runtime.Serialization;

namespace ERWon
{

	/// <summary>Documentos que o aplicativo pode editar.</summary>
	[ Serializable ]
	public class Document : DocElement, IDeserializationCallback, ISerializable, IDocument
	{
		/// <summary>
		/// Caminho do arquivo (XML) do documento
		/// </summary>
		protected string m_strPath;
		[Browsable (false)]
		public string strPath 
		{ 
			get { return m_strPath; } 
			set 
			{ 
				m_strPath = value;
				if (Notify_PathChanged != null)
					Notify_PathChanged (m_strPath);
			} 
		}


		/// <summary>Indica se h mudanas (no salvas) no documento</summary>
		protected bool m_fChanged = false;

		[Browsable (false)]
		public bool Changed 
		{ 
			get { return m_fChanged; } 
			set { 
				// Se o valor alterou, notificar...
				if (value != m_fChanged) 
				{
					m_fChanged = value; 
					if (Notify_Changed != null)
						Notify_Changed (value);
				}
			} 
		}


		/// <summary>Eventos (na verdade delegates) que notificam o MainForm de mudanas no documento</summary>
		public delegate void SetChanged (bool fChanged);
		public SetChanged Notify_Changed;

		public delegate void SetPath (string strPath);
		public SetPath Notify_PathChanged;


		/// <summary>Lista de elementos do Documento - Boxes e Lines</summary>
		protected ArrayList m_Boxes = new System.Collections.ArrayList ();
		protected ArrayList m_Lines = new System.Collections.ArrayList ();

		/// <summary>Titulo</summary>
		public string m_strTitle;

		/// <summary>Comentrios</summary>
		public string m_strComments;

		/// <summary>Visualizao a ser atualizada.
		/// Quando houver modelo fisico E logico, m_View dever ser uma lista de IViews. Por en
		/// </summary>
		public IView m_View;

		/// <summary>Background</summary>
		public Color m_BackgroundColor = Color.White;


		/// <summary>Construtor</summary>
		public Document ()
		{
			m_fChanged = false;
		}

		/// <summary>Construtor de Serializao</summary>
		protected Document (SerializationInfo Info, StreamingContext Context)
		{
			m_Boxes = Info.GetValue ("Boxes", typeof (object)) as ArrayList;
			m_Lines = Info.GetValue ("Lines", typeof (object)) as ArrayList;
			m_strTitle = Info.GetString ("Title");
			m_strComments = Info.GetString ("Comments");
			m_BackgroundColor = Util.StringToColor (Info.GetString ("BackgroundColor"));
		}
		/// <summary>Deserializao</summary>
		public override void GetObjectData (SerializationInfo Info, StreamingContext Context)
		{
			base.GetObjectData (Info, Context);
			Info.AddValue ("Boxes", m_Boxes);
			Info.AddValue ("Lines", m_Lines);
			Info.AddValue ("Title", m_strTitle);
			Info.AddValue ("Comments", m_strComments);
			Info.AddValue ("BackgroundColor", Util.ColorToString (m_BackgroundColor));
		}

		/// <summary>Aps deserializao acabar, algumas inicializaes restantes...</summary>
		/// TODO - trocar por variavel pre-inicializada? Testar.
		public void OnDeserialization (object foo)
		{
			m_fChanged = false;
			foreach (Box EachBox in m_Boxes)
				EachBox.Document = this;
			foreach (Line EachLine in m_Lines)
				EachLine.Document = this;
		}



		/// <summary>Atualizar todas visualizaes do documento.
		/// Por enquanto  uma s. Um dia haver modelos lgico E fsico.
		/// </summary>
		// TODO - quando houver modelo fisico E logico, m_View dever ser uma lista de IViews.
		public void UpdateAllViews ()
		{
			m_View.Repaint();
		}


		/// <summary>Seleciona todos elementos do documento</summary>
		/// Obs: o proprio documento no  selecionado. Se quiser, segure o ctrl e clique.
		/// </summary>
		public void SelectAll (DocSubset Subset)
		{
			foreach (Box EachBox in Boxes)
				Subset.Select (EachBox);

			foreach (Line EachLine in Lines)
				Subset.Select (EachLine);
		}


		/// <summary>Implementando Interface IDocument</summary>
		[Browsable (false)]
		public ArrayList Boxes { get { return m_Boxes; } }

		/// <summary>Implementando Interface IDocument</summary>
		[Browsable (false)]
		public ArrayList Lines { get { return m_Lines; } }

		/// <summary>Titulo</summary>
		[Description ("Title of this document"), Category ("Misc")]
		public string Title { get { return m_strTitle; } set { m_strTitle = value; } }

		/// <summary>Comentarios</summary>
		[Description ("Feel free to write whatever you wish to write"), Category ("Misc")]
		public string Comments { get { return m_strComments; } set { m_strComments = value; } }

		/// <summary>Estatisticas sobre o documento...</summary>
		[Description ("Number of entities in the model"), Category ("Objects")]
		public uint Entities { get { return (uint) m_Boxes.Count; } }

		/// <summary>Estatisticas sobre o documento...</summary>
		[Description ("Number of relationships in the model"), Category ("Objects")]
		public uint Relationships { get { return (uint) m_Lines.Count; } }

		/// <summary>Background</summary>
		[Description ("Background Color"), Category ("Appearance")]
		public Color Background { get { return m_BackgroundColor; } set { m_BackgroundColor = value; } }




		/// <summary>Adiciona elemento do tipo IBox ao documento</summary>
		public void Add (IBox NewBox)
		{
			m_Boxes.Add (NewBox);
			ParentDocument().Changed = true;
			UpdateAllViews ();
		}


		/// <summary>Adiciona elemento do tipo ILine ao documento</summary>
		/// TODO - isso  resqucio de quando era VISIO-STYLE.
		public void Add (ILine NewLine)
		{
			m_Lines.Add (NewLine);
			ParentDocument().Changed = true;
			UpdateAllViews ();
		}


		/// <summary>Adiciona um DocElement qualquer ao documento</summary>
		public void Add (DocElement Thing)
		{
			Box NewBox = Thing as Box;
			Line NewLine = Thing as Line;

			if (NewBox != null)
			{
				NewBox.Parent = this;
				Add (NewBox as IBox);				
			}

			if (NewLine != null)
			{
				NewLine.Parent = this;
				Add (NewLine as ILine);				
			}
		}


		/// <summary>Operaes de Clipboard: Adicionar os elementos ao documento</summary>
		/// <param name="objects">Elementos do tipo DocElement a serem colados no documento</param>
		public void Paste (ArrayList objects)
		{
			foreach (DocElement element in objects)
			{
				if (element != null)
				{
					Add (element);
				}
			}
			ParentDocument().Changed = true;
		}		


		/// <summary>Remove uma lista de elementos do documento</summary>
		/// <param name="DoomedObjects">Elementos do tipo DocElement para remover do documento</param>
		/// <returns>Objetos apagados como resultado desta operao</returns>
		public ArrayList Delete (ArrayList DeletedElements)
		{
			ArrayList CollateralDeletes = new ArrayList ();

			foreach (DocElement elemento in DeletedElements)
			{
				Box DeletedBox = elemento as Box;
				Line DeletedLine = elemento as Line;

				// Se apagou um Box, consequentemente algumas Lines tambm sero apagadas.
				if (DeletedBox != null)
					CollateralDeletes.AddRange (Delete (DeletedBox));

				if (DeletedLine != null)
					Delete (DeletedLine);
			}

			ParentDocument().Changed = true;

			return CollateralDeletes;
		}


		/// <summary>Apaga um Box</summary>
		public ArrayList Delete (IBox DeletedBox)
		{
			m_Boxes.Remove (DeletedBox);
			UpdateAllViews ();
			ParentDocument().Changed = true;
			//TODO - apagar as Lines que dependiam do Box, e retornar array.
			return null;
		}


		/// <summary>Apaga uma Line</summary>
		public void Delete (ILine DeletedLine)
		{
			m_Lines.Remove (DeletedLine);
			ParentDocument().Changed = true;
			UpdateAllViews ();			
		}



		/// <summary>
		/// Move todos os Boxes do documento.
		/// TODO: Usei para permitir scroll abaixo do X=0 ou Y=0, mas no ficou muito bom... :(
		/// </summary>
		public override void Drag (int iDeltaX, int iDeltaY)
		{
			foreach (Box EachBox in m_Boxes)
			{
				EachBox.Drag(iDeltaX, iDeltaY);
			}
		}

		/// <summary>Desenha o documento. Se estiver selecionado, tem borda.</summary>
		public void Draw (System.Drawing.Graphics g, Control c)
		{
			if (this.Selected)
			{
				Rectangle docrectangle = new Rectangle(Convert.ToInt32(c.ClientRectangle.X+5),Convert.ToInt32(c.ClientRectangle.Y+5),Convert.ToInt32(c.ClientRectangle.Width-10),Convert.ToInt32(c.ClientRectangle.Height-10));
				g.DrawRectangle(new Pen(Color.Black, 3), docrectangle);
			}
		}




	}
}

