/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe Line - Toda linha (conector) deriva desta classe,
 * inclusive relacionamentos
 ************************************************************************/

using System;
using System.ComponentModel;
using System.Drawing;
using System.Runtime.Serialization;

namespace ERWon
{
	/// <summary>Define aparencia e formato das Linhas usadas no aplicativo
	/// </summary>
	[ Serializable ]
	public class Line : DocElement, ILine, ISerializable
	{
		/// <summary>Box de origem</summary>
		protected Box m_Source;

		/// <summary>Box de destino</summary>
		protected Box m_Destination;

		/// <summary>Ponto de conexo com o Box origem</summary>
		public uint m_SourceConnectionID;

		/// <summary>Ponto de conexo com o Box destino</summary>
		public uint m_DestinationConnectionID;

		/// <summary>Se a Line no tiver Box de origem, este ponto ser sua coordenada.</summary>
		private Point m_SourcePoint = new Point ();

		/// <summary>Se a Line no tiver Box de destino, este ponto ser sua coordenada.</summary>
		private Point m_DestinationPoint = new Point ();

		/// <summary>Box origem</summary>
		[Browsable (false)]
		public IBox Source 
		{
			get { return (IBox) m_Source; }
			set { m_Source = value as Box; }
		}

		/// <summary>Box destino</summary>
		[Browsable (false)]
		public IBox Destination 
		{
			get {return (IBox) m_Destination;}
			set {m_Destination = value as Box;}
		}	

		/// <summary>Ponto de origem. Pode ser um ponto solto, ou um ponto de um Box.</summary>
		[Browsable (false)]
		public Point SourcePoint
		{
			get
			{
				if ((m_Source != null) && (m_SourceConnectionID > 0))
					return m_Source.GetSpecialPoint (m_SourceConnectionID);

				return m_SourcePoint;
			}

			set
			{
				m_SourcePoint = value;
			}
		}

		/// <summary>Ponto de destino. Pode ser um ponto solto, ou um ponto de um Box.</summary>
		[Browsable (false)]
		public Point DestinationPoint
		{
			get
			{
				if ((m_Destination != null) && (m_DestinationConnectionID > 0))
					return m_Destination.GetSpecialPoint (m_DestinationConnectionID);

				return m_DestinationPoint;
			}

			set
			{
				m_DestinationPoint = value;
			}
		}

		/// <summary>Nome do Box origem</summary>
		[Description ("Name of the Box that this Line comes from")]
		public string SourceName { get { if (m_Source != null) return m_Source.Text; else return ""; } }

		/// <summary>Nome do Box destino</summary>
		[Description ("Name of the Box that this Line goes to")]
		public string DestinationName { get { if (m_Destination != null) return m_Destination.Text; else return ""; } }


		Color m_Color;
		int m_Width;

		[Description ("Line Color"), Category ("Appearance")]
		public Color Color 
		{
			get { return m_Color; }
			set { m_Color = value; }
		}

		[Description ("Line Width"), Category ("Appearance")]
		public int Width
		{
			get { return m_Width; }
			set { m_Width = value; }
		}


		public Line ()
		{
			m_Color = System.Drawing.Color.Black;
		}
		protected Line (SerializationInfo Info, StreamingContext Context) : base (Info, Context)
		{
			m_Source = Info.GetValue ("Source", typeof (object)) as Box;
			m_Destination = Info.GetValue ("Destination", typeof (object)) as Box;			

			m_SourceConnectionID = Info.GetUInt32 ("SourceConnectionID");
			m_SourcePoint.X = Info.GetInt32 ("SourcePointX");
			m_SourcePoint.Y = Info.GetInt32 ("SourcePointY");

			m_DestinationConnectionID = Info.GetUInt32 ("DestinationConnectionID");
			m_DestinationPoint.X = Info.GetInt32 ("DestinationPointX");
			m_DestinationPoint.Y = Info.GetInt32 ("DestinationPointY");			

			m_Color = Util.StringToColor (Info.GetString ("Color"));
			m_Width = Info.GetInt32 ("Width");			
		}
		public override void GetObjectData (SerializationInfo Info, StreamingContext Context)
		{
			base.GetObjectData (Info, Context);
			Info.AddValue ("Source", m_Source);
			Info.AddValue ("Destination", m_Destination);				

			Info.AddValue ("SourceConnectionID", m_SourceConnectionID);
			Info.AddValue ("SourcePointX", m_SourcePoint.X);
			Info.AddValue ("SourcePointY", m_SourcePoint.Y);

			Info.AddValue ("DestinationConnectionID", m_DestinationConnectionID);
			Info.AddValue ("DestinationPointX", m_DestinationPoint.X);
			Info.AddValue ("DestinationPointY", m_DestinationPoint.Y);			

			Info.AddValue ("Color", Util.ColorToString (m_Color));
			Info.AddValue ("Width", m_Width);
		}


		/// <summary>Seta o Box origem</summary>
		/// <param name="SourceBox"></param>
		/// <param name="ConnectionID"></param>
		public void SetSource (Box SourceBox, uint ConnectionID)
		{
			m_Source = SourceBox;
			m_SourceConnectionID = ConnectionID;
		}

		/// <summary>Seta o Box destino</summary>
		public void SetDestination (Box DestinationBox, uint ConnectionID)
		{
			m_Destination = DestinationBox;
			m_DestinationConnectionID = ConnectionID;
		}

		/// <summary>Cria uma Line nas coordenadas dadas, sem conexo a Boxes.</summary>
		public void Init (int X, int Y)
		{
			m_SourcePoint = new Point (X - 10, Y - 10);
			m_DestinationPoint = new Point (X + 10, Y + 10);
		}




		/// <summary>Determina se o ponto est prximo da Line</summary>
		/// Funciona calculando distancia entre ponto e reta.
		/// d = Abs( (x2-x1)(y1-y0)-(x1-x0)(y2-y1) )
		///     ------------------------------------
		///         sqr( (x2-x1)^2 + (y2-y1)^2 )
		///         
		/// http://mathworld.wolfram.com/Point-LineDistance2-Dimensional.html
		/// aonde P=(x0,y0), reta=(x1,y1)-(x2,y2)
		public bool Hit (int iX, int iY)
		{
			Point Source = SourcePoint;
			Point Destination = DestinationPoint;

			if (Util.HitTest (SourcePoint, iX, iY))
				return true;

			if (Util.HitTest (DestinationPoint, iX, iY))
				return true;

			int iSourceX = Source.X;
			int iSourceY = Source.Y;

			int iDestinationX = Destination.X;
			int iDestinationY = Destination.Y;

			double e = Math.Abs( (iDestinationX-iSourceX)*(iSourceY-iY) - (iSourceX-iX)*(iDestinationY-iSourceY));
            double f = Math.Sqrt( Math.Pow((iDestinationX-iSourceX),2) + Math.Pow((iDestinationY-iSourceY),2));
			double d=e/f;

			return (d<Util.hittest_precision);
		}



		/// <summary>Desenha o elemento da origem ao destino</summary>
		public void Draw (System.Drawing.Graphics g)
		{
			Pen Color = null;
			
			if (Selected)
				Color = new System.Drawing.Pen (Util.Highlight (m_Color), m_Width + 3);
			else
				Color = new System.Drawing.Pen (m_Color, m_Width);

			Point Source = SourcePoint;
			Point Destination = DestinationPoint;
			g.DrawLine (Color, Source.X, Source.Y, Destination.X, Destination.Y);

			DrawSpecialPoint (g, Source.X, Source.Y);
			DrawSpecialPoint (g, Destination.X, Destination.Y);
		}


		/// <summary>Cria cpia do elemento para uso em clipboard</summary>
		public override DocElement CreateClipboardCopy () 
		{
			Line Copy = base.CreateClipboardCopy () as Line;

			Copy.m_Source = m_Source;
			Copy.m_Destination = m_Destination;

			Copy.m_SourceConnectionID = m_SourceConnectionID;
			Copy.m_DestinationConnectionID = m_DestinationConnectionID;

			Copy.Color = Color;
			Copy.Width = Width;

			return Copy as DocElement;
		}

	}
}
