/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe Commands -
 * Todas aes desfazveis esto aqui, derivam de Command, e so
 * gerenciadas pelo CommandProcessor
 ************************************************************************/

using System.Collections;
using System.Windows.Forms;
using System.Drawing;

namespace ERWon
{
	/// <summary>Comando de adicionar um Box</summary>
	public class AddBox : Command
	{
		private Box m_Box;
		private object[] m_OldPropertySelection;
		private MainForm m_MainForm;

		public override Document ParentDocument { get { return m_Box.ParentDocument(); } }

		/// <summary>Construtor</summary>
		/// <param name="NewBox">Box para adicionar</param>
		/// <param name="MainForm">Janela principal do Aplicativo</param>
		/// <param name="OldPropertySelection">Array de PropertyGrid.Selection</param>
		public AddBox (Box NewBox, MainForm MainForm, object[] OldPropertySelection)
		{
			m_strShortDescription = "Add Box";
			m_fCanUndo = true;

			m_Box = NewBox;

			m_MainForm = MainForm;
			m_OldPropertySelection = OldPropertySelection;
		}


		/// <summary>Executa o comando</summary>
		public override void Execute ()
		{
			Document Parent = m_Box.Document as Document;
            Parent.Add (m_Box as DocElement);

			m_MainForm.m_SelectionSet.Empty ();
			m_MainForm.m_SelectionSet.Select (m_Box);

			(m_Box.Document as Document).UpdateAllViews ();
		}

		/// <summary>Desfaz o comando</summary>
		public override void Undo ()
		{
			Document Parent = m_Box.Document as Document;
			Parent.Delete (m_Box);

			m_MainForm.m_SelectionSet.Empty ();
			foreach (object ThisBetterBeAnElement in m_OldPropertySelection)
			{
				DocElement Thing = ThisBetterBeAnElement as DocElement;

				if (Thing != null)
				{
					m_MainForm.m_SelectionSet.Select (Thing);
					(Thing.Document as Document).UpdateAllViews ();
				}
			}
		}
	}

	/// <summary>Comando de adicionar uma Line</summary>
	public class AddLine : Command
	{
		private Line m_Line;
		private object[] m_OldPropertySelection;
		private MainForm m_MainForm;

		public override Document ParentDocument { get { return m_Line.ParentDocument(); } }

		public AddLine (Line NewLine, MainForm MainForm, object[] OldPropertySelection)
		{
			m_strShortDescription = "Add Line";
			m_fCanUndo = true;

			m_Line = NewLine;

			m_MainForm = MainForm;
			m_OldPropertySelection = OldPropertySelection;
		}

		/// <summary>Executa o comando</summary>
		public override void Execute ()
		{
			Document Parent = m_Line.Document as Document;
			Parent.Add (m_Line as DocElement);

			m_MainForm.m_SelectionSet.Empty ();
			m_MainForm.m_SelectionSet.Select (m_Line);

			(m_Line.Document as Document).UpdateAllViews ();
		}

		/// <summary>Desfaz o comando</summary>
		public override void Undo ()
		{
			Document Parent = m_Line.Document as Document;
			Parent.Delete (m_Line);

			// Restaura o SelectionSet anterior
			m_MainForm.m_SelectionSet.Empty ();
			foreach (object ThisBetterBeSelectable in m_OldPropertySelection)
			{
				DocElement Thing = ThisBetterBeSelectable as DocElement;

				if (Thing != null)
				{
					m_MainForm.m_SelectionSet.Select (Thing);
					(Thing.Document as Document).UpdateAllViews ();
				}
			}
		}
	};


	/// <summary>Comando de apagar elementos do documento</summary>
	public class EditDelete : Command
	{
		private ArrayList m_DeletedObjects = new ArrayList ();
		private Document m_Parent;
		private DocSubset m_Selection;
		private ArrayList m_Objects;

		public override Document ParentDocument { get { return m_Parent.ParentDocument(); } }

		public EditDelete (Document Parent, DocSubset Selection)
		{
			m_strShortDescription = "Delete";
			m_fCanUndo = true;

			m_Parent = Parent;
			m_Selection = Selection;
			m_Objects = new ArrayList (m_Selection.m_Elements);
		}

		/// <summary>Executa o comando
		/// Apaga os objetos selecionados, mas salva para caso isso seja desfeito.
		/// </summary>
		public override void Execute ()
		{
			foreach (DocElement Thing in m_Objects)
			{
				Box DeletedBox = Thing as Box;
				Line DeletedLine = Thing as Line;

				if (DeletedBox != null)
				{
					Document Parent = DeletedBox.Document as Document;
					if (Parent == m_Parent)
					{
						ArrayList CollateralDamage = Parent.Delete (DeletedBox);
						//TODO - salvar CollateralDamage ?
						//m_DeletedObjects.AddRange (CollateralDamage);
					}

					m_DeletedObjects.Add (Thing);
				}

				if (DeletedLine != null)
				{
					Document Parent = DeletedLine.Document as Document;
					if (Parent == m_Parent)
						Parent.Delete (DeletedLine);

					m_DeletedObjects.Add (Thing);
				}
			}

			m_Selection.Remove (m_DeletedObjects);
		}

		/// <summary>Desfaz o comando</summary>
		public override void Undo ()
		{
			foreach (DocElement Thing in m_DeletedObjects)
			{
				Box DeletedBox = Thing as Box;

				if (DeletedBox != null)
				{
					Document Parent = DeletedBox.Document as Document;
					if (Parent == m_Parent)
						Parent.Add (DeletedBox as DocElement);

				}
			}

			foreach (DocElement Thing in m_DeletedObjects)
			{
				Line DeletedLine = Thing as Line;

				if (DeletedLine != null)
				{					
					Document Parent = DeletedLine.Document as Document;
					if (Parent == m_Parent)
						Parent.Add (DeletedLine as DocElement);

				}
			}

			m_Selection.Empty ();

			foreach (DocElement Thing in m_DeletedObjects)
			{
				m_Selection.Select (Thing);
			}
		}
	}



	/// <summary>Comando de colar</summary>
	public class EditPaste : Command
	{
		private Document m_Parent;
		private ArrayList m_NewObjects;
		public override Document ParentDocument { get { return m_Parent.ParentDocument(); } }

		public EditPaste (Document Parent, ArrayList NewObjects)
		{
			m_fCanUndo = true;
			m_strShortDescription = "Paste";

			m_Parent = Parent;
			m_NewObjects = NewObjects;
		}

		/// <summary>Executa o comando</summary>
		public override void Execute ()
		{
			m_Parent.Paste (m_NewObjects);
		}

		/// <summary>Desfaz o comando</summary>
		public override void Undo ()
		{
			m_Parent.Delete (m_NewObjects);
		}
	}

	/// <summary>Comando de recortar. Identico ao de apagar, s muda o nome.
	/// </summary>
	public class EditCut : EditDelete
	{
		public EditCut (Document Parent, DocSubset Selection) : base (Parent, Selection)
		{
			m_strShortDescription = "Cut";
			m_fCanUndo = true;
		}
	}

	/// <summary>Comando acontece quando alguma propriedade do propertygrid 
	/// alterada para o(s) objeto(s) selecionado(s).
	/// </summary>
	/// TODO - salvar o valor antigo de todos objetos sendo modificados. Ser possvel?
	public class PropertyChange : Command
	{
		private object[] m_Objects;
		private GridItem m_Property;
		private object m_OldValue;
		private object m_NewValue;

		public override string ShortDescription { get { return "Change " + m_Property.PropertyDescriptor.Name; } }

		public PropertyChange (object[] Objects, GridItem Property, object NewValue, object OldValue)
		{
			m_fCanUndo = true;

			m_Objects = Objects;
			m_Property = Property;
			m_OldValue = OldValue;
			m_NewValue = NewValue;
		}

		/// <summary>Executa o comando</summary>
		public override void Execute ()
		{
			foreach (object EachObject in m_Objects)
			{
				try { m_Property.PropertyDescriptor.SetValue (EachObject, m_NewValue); }
				catch (System.Exception) {}
			}

			UpdateViews ();
		}

		/// <summary>Desfaz o comando</summary>
		public override void Undo ()
		{
			foreach (object EachObject in m_Objects)
				m_Property.PropertyDescriptor.SetValue (EachObject, m_OldValue);

			UpdateViews ();
		}

		/// <summary>Atualiza as visualizaes dos documentos relacionados.
		/// </summary>
		/// TODO - est ineficiente...
		protected void UpdateViews ()
		{
			foreach (object EachObject in m_Objects)
			{
				// Examinando todas classes que podem ter sido alteradas..
				Box EachBox = EachObject as Box;
				Line EachLine = EachObject as Line;
				Document EachDoc = EachObject as Document;
				
				if (EachBox != null)
					EachDoc = EachBox.Document as Document;
				
				if (EachLine != null)
					EachDoc = EachLine.Document as Document;

				if (EachDoc != null)
					EachDoc.UpdateAllViews ();
			}
		}
	}

	/// <summary>Comandos de Drag (movimentao)
	/// Funcionam assim:
	/// - Evento DragStart - salva a posio do mouse
	/// - DragContinue - enquanto o mouse vai se movendo
	/// - DragEnd - quando o usurio solta boto do mouse, a sim o comando completa.
	/// </summary>
	public class Drag : Command
	{
		protected Document m_Parent;
		protected ArrayList m_Objects;
		protected bool m_fUsed;
		protected bool m_fInvokedOnce;
		protected int m_iStartX, m_iStartY;
		protected int m_iDeltaX, m_iDeltaY;
		protected int m_iLastX, m_iLastY;

		public override Document ParentDocument { get { return m_Parent.ParentDocument(); } }

		/// <summary>
		/// Indica se o Drag se sucedeu mesmo ou se apenas comecou (clique sem drag).
		/// </summary>
		public bool Used { get { return m_fUsed; } }

		public Drag (Document Parent, ArrayList Objects, int iStartX, int iStartY)
		{
			m_strShortDescription = "Drag";
			m_fCanUndo = true;
			m_Parent = Parent;

			m_Objects = new ArrayList ();
			m_fUsed = false;
			m_fInvokedOnce = false;
			m_iLastX = m_iStartX = iStartX;
			m_iLastY = m_iStartY = iStartY;

			foreach (DocElement EachThing in Objects)
				m_Objects.Add (EachThing);
		}

		/// <summary>
		/// Enquanto o mouse se move, atualiza a UI
		/// </summary>
		public void Continue (int iNewX, int iNewY)
		{
			// Qual foi o deslocamento total?
			int iDeltaX = iNewX - m_iLastX;
			int iDeltaY = iNewY - m_iLastY;

			m_iLastX = iNewX;
			m_iLastY = iNewY;

			// Mover objetos selecionados
			DragAll (iDeltaX, iDeltaY);

			m_iDeltaX = iNewX - m_iStartX;
			m_iDeltaY = iNewY - m_iStartY;

			m_fUsed = true;
		}

		/// <summary>Movimenta os objetos selecionados.</summary>
		private void DragAll (int iDeltaX, int iDeltaY)
		{
			if ((iDeltaX == 0) && (iDeltaY == 0))
				return;

			foreach (DocElement EachObject in m_Objects)
				EachObject.Drag (iDeltaX, iDeltaY);

			ParentDocument.Changed = true;
            m_Parent.UpdateAllViews ();
		}

		/// <summary>Executa o comando</summary>
		public override void Execute ()
		{
			/// Na primeira execuo, o comando NO deve chamar o DragAll, pois
			/// os proprios objetos j fora movidos no mouse (e no Drag_Continue).
			/// Somente nas proximas execues (REDO)  que o DragAll deve ser chamado.
			if (m_fInvokedOnce)
				DragAll (m_iDeltaX, m_iDeltaY);
			else
				m_fInvokedOnce = true;
		}

		/// <summary>Desfaz o comando</summary>
		public override void Undo ()
		{
			DragAll (-m_iDeltaX, -m_iDeltaY);
		}
	}

	/// <summary>Comando de redimensionar Box</summary>
	public class Resize : Command
	{
		protected Document m_Parent;
		protected bool m_fUsed;
		protected Box m_Box;
		protected Box_Corner m_BoxCorner;
		protected int m_iNewX, m_iNewY;

		protected int m_oldX0, m_oldY0, m_oldX1, m_oldY1;
		protected int m_iStartX, m_iStartY;

		public override Document ParentDocument { get { return m_Parent.ParentDocument(); } }

		/// <summary>
		/// Indica se o comando foi de fato usado ou se apenas foi iniciado
		/// (clicado) e nada foi feito.
		/// </summary>
		public bool Used { get { return m_fUsed; } }

		public Resize (Document Parent, Box ThisBox, Box_Corner BoxCorner, int iStartX, int iStartY)
		{
			m_strShortDescription = "Resize";
			m_fCanUndo = true;
			m_Parent = Parent;
			m_Box = ThisBox;
			m_BoxCorner = BoxCorner;

			m_fUsed = false;

			this.m_oldX0 = m_Box.X0;
			this.m_oldY0 = m_Box.Y0;
			this.m_oldX1 = m_Box.X1;
			this.m_oldY1 = m_Box.Y1;
			this.m_iStartX = iStartX;
			this.m_iStartY = iStartY;
		}

		/// <summary>Enquanto o mouse se move, atualiza a UI</summary>
		public void Continue (int iNewX, int iNewY)
		{
			m_iNewX = iNewX;
			m_iNewY = iNewY;

			int iDeltaX = m_iNewX - m_iStartX;
			int iDeltaY = m_iNewY - m_iStartY;

			// Se o Corner (extremo) sendo redimensionado for algum do topo, redimensionar Y0...
			if (m_BoxCorner == Box_Corner.Top || m_BoxCorner == Box_Corner.TopLeft || m_BoxCorner == Box_Corner.TopRight)
				if (m_Box.Height - iDeltaY >= m_Box.m_MinHeight) m_Box.Y0 = m_oldY0 + iDeltaY;
			// Se o Corner (extremo) sendo redimensionado for algum da base, redimensionar Y1...
			if (m_BoxCorner == Box_Corner.Bottom || m_BoxCorner == Box_Corner.BottomLeft || m_BoxCorner == Box_Corner.BottomRight)
				if (m_Box.Height + iDeltaY >= m_Box.m_MinHeight) m_Box.Y1 = m_oldY1 + iDeltaY;
			// Se o Corner (extremo) sendo redimensionado for algum da esquerda, redimensionar X0...
			if (m_BoxCorner == Box_Corner.TopLeft || m_BoxCorner == Box_Corner.Left || m_BoxCorner == Box_Corner.BottomLeft)
				if (m_Box.Width - iDeltaX >= m_Box.m_MinWidth) m_Box.X0 = m_oldX0 + iDeltaX;
			// Se o Corner (extremo) sendo redimensionado for algum da direita, redimensionar X1...
			if (m_BoxCorner == Box_Corner.TopRight || m_BoxCorner == Box_Corner.Right || m_BoxCorner == Box_Corner.BottomRight)
				if (m_Box.Width + iDeltaX >= m_Box.m_MinWidth) m_Box.X1 = m_oldX1 + iDeltaX;
		
			m_fUsed = true;

			m_Parent.UpdateAllViews ();
		}

		/// <summary>Executa o comando</summary>
		public override void Execute ()
		{
			Continue(m_iNewX, m_iNewY);
			ParentDocument.Changed = true;
			m_Parent.UpdateAllViews ();
		}

		/// <summary>Desfaz o comando</summary>
		public override void Undo ()
		{
			m_Box.X0 = this.m_oldX0;
			m_Box.Y0 = this.m_oldY0;
			m_Box.X1 = this.m_oldX1;
			m_Box.Y1 = this.m_oldY1;

			ParentDocument.Changed = true;
			m_Parent.UpdateAllViews ();
		}
	}

}
