/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe DocSubset
 * Subcojunto dos elementos de um documento.
 * Basicamente,  usado nas operaes de clipboard.
 * Tambm mantm o PropertyGrid atualizado, e cria conjuntos de elementos
 * duplicados, restaurando os relacionamentos entre os elementos originais.
 ************************************************************************/

using System;
using System.Collections;
using System.Runtime.Serialization;

namespace ERWon
{
	/// <summary>Derivado de ArrayList, implementa uma nova funo
	/// que recebe um elemento (de uma lista clonada), e percorre
	/// a lista original para obter o elemento original que deu
	/// origem ao elemendo clonado
	/// </summary>
	public class DuplicateArrayList : ArrayList
	{
		public DocElement GetCopyOf (DocElement Original)
		{
			foreach (DocElement Duplicate in this)
				if (Duplicate.OriginalElement == Original)
					return Duplicate;
			return null;
		}
	}

	/// <summary>Subcojunto dos elementos de um documento.
	/// Basicamente,  usado nas operaes de clipboard.
	/// 
	/// O aplicativo mantm um subconjunto de elementos com todos
	/// objetos que esto selecionados na interface. Os mtodos Select
	/// e Deselect manipulam estes estados (de seleo) dos elementos no
	/// subconjunto.
	/// </summary>
	[Serializable]
	public class DocSubset : ISerializable
	{
		/// <summary>Subconjunto de elementos</summary>
		public ArrayList m_Elements = new ArrayList ();

		/// <summary>Nmero de elementos no subconjunto</summary>
		public uint Size { get { return (uint) m_Elements.Count; } }

		/// <summary>Associar o subconjunto com um controle propertygrid</summary>
		private System.Windows.Forms.PropertyGrid m_Properties;

		public DocSubset (System.Windows.Forms.PropertyGrid Properties)
		{
			m_Properties = Properties;
		}

		/// <summary>Construtor de deserializao</summary>
		protected DocSubset (SerializationInfo Info, StreamingContext Context)
		{
			m_Elements = Info.GetValue ("SelectionSet", typeof (ArrayList)) as ArrayList;
		}

		/// <summary>Serializao</summary>
		/// Se os prprios objetos forem serializados, todo
		/// o grafo de dependencias vai ser serializado j que
		/// o cdigo de serializao segue todas referencias. Ento
		/// criamos uma lista de objetos duplicados, e esta lista que 
		/// copiada.
		/// </summary>
		/// TODO - colocar tambm no clipboard em um formato texto, para
		/// que possa ser "debugado" colando no notepad.
		public virtual void GetObjectData (SerializationInfo Info, StreamingContext Context)
		{
			//ArrayList Duplicates = CreateDuplicates ();
			//Info.AddValue ("SelectionSet", Duplicates);
		}


		/// <summary>
		/// Cria uma lista de objetos duplicados que preserva
		/// os relacionamentos entre os objetos no conjunto cpia.
		/// A lista de duplicados  identica a original, com
		/// excesso de que referencias a objetos que no estiverem
		/// no conjunto (selecionado) se tornaro nulas.
		/// </summary>
		public ArrayList CreateDuplicates (DocElement ActiveDocument)
		{
			DuplicateArrayList Duplicates = new DuplicateArrayList ();
			ArrayList Originals = new ArrayList (m_Elements);

			// Copiando os Boxes
			foreach (DocElement Thing in Originals)
			{
				Box OriginalBox = Thing as Box;

				if (OriginalBox != null)
				{
					if (!OriginalBox.ChildOf (ActiveDocument))
						continue;

					// Duplicando o Box e adicionando a lista.
					Box NewBox = OriginalBox.CreateClipboardCopy () as Box;
					Duplicates.Add (NewBox);
				}
			}

			// Removendo Boxes copiados da lista original
			foreach (DocElement NewThing in Duplicates)
				if (NewThing != null)
					Originals.Remove (NewThing.OriginalElement);


			// Copiando as Lines
			foreach (DocElement Thing in Originals)
			{
				Line OriginalLine = Thing as Line;

				if (OriginalLine != null)
				{
					if (!OriginalLine.ChildOf (ActiveDocument))
						continue;

					/// Duplicando a Line, verificando se referencias tambm foram
					/// copiadas, e adicionando a lista.

					Line NewLine = OriginalLine.CreateClipboardCopy () as Line;

					Box NewSource = Duplicates.GetCopyOf (NewLine.Source as Box) as Box;
					if (NewSource != null)
						NewLine.Source = NewSource;
					else
						NewLine.Source = null;

					Box NewDestination = Duplicates.GetCopyOf (NewLine.Destination as Box) as Box;
					if (NewDestination != null)
						NewLine.Destination = NewDestination;
					else
						NewLine.Destination = null;

					Duplicates.Add (NewLine);
				}
			}

			/// Removendo Lines copiadas da lista original
			foreach (DocElement NewThing in Duplicates)
				if (NewThing != null)
					Originals.Remove (NewThing.OriginalElement);

			/// Verificar se dos elementos no copiados, se algum
			///  descendente do documento ativo.
			foreach (DocElement Thing in Originals)
			{
				System.Diagnostics.Debug.Assert(!Thing.ChildOf (ActiveDocument));
			}

			// Apaga as referencias pros originais - j no  mais necessria
			foreach (DocElement Duplicate in Duplicates)
				Duplicate.OriginalElement = null;

			// Transformar pra ArrayList, pra passar pro clipboard.
			return new ArrayList (Duplicates);
		}

		/// <summary>Adiciona um elemento ao conjunto selecionado</summary>
		public void Select (DocElement Thing)
		{
			Thing.Select ();
			if (!m_Elements.Contains (Thing))
				m_Elements.Add (Thing);
			AddToPropertyGrid (Thing);
		}


		/// <summary>Remove um elemento do conjunto selecionado</summary>
		public void Deselect (DocElement Thing)
		{
			Thing.Deselect ();
			m_Elements.Remove (Thing);
			RemoveFromPropertyGrid (Thing);
		}

		
		/// <summary>Remove uma lista de elementos do conjunto selecionado</summary>
		public void Remove (ArrayList Objects)
		{
			foreach (DocElement Thing in Objects)
			{
				if (Thing.Selected)
					Thing.Deselect ();
				m_Elements.Remove (Thing);
				RemoveFromPropertyGrid (Thing);
			}
		}

		
		/// <summary>Esvazia o conjunto selecionado (deseleciona tudo)</summary>
		public void Empty ()
		{
			foreach (DocElement Thing in m_Elements)
				while (Thing.Selected)
					Thing.Deselect ();

			m_Elements.Clear ();

			if (m_Properties != null)
				m_Properties.SelectedObject = null;
		}

		
		/// <summary>Testa se o conjunto selecionado  vazio</summary>
		public bool IsEmpty ()
		{
			return (m_Elements.Count == 0);
		}

		/// <summary>Adiciona um elemento ao array SelectedObjects do Property Grid</summary>
		private void AddToPropertyGrid (DocElement Thing)
		{
			/// Talvez o DocSubset seja usado sem property grid...
			if (m_Properties == null)
				return;

			int Length = m_Properties.SelectedObjects.Length;

			if (Length < 1)
			{
				m_Properties.SelectedObject = Thing; 
				return;
			}

			object [] NewSet = new object[Length + 1];

			int i = 0;
			foreach (object EachObject in m_Properties.SelectedObjects)
			{
				DocElement EachThing = EachObject as DocElement;

				if (EachThing != null)
				{
					NewSet[i] = EachThing;
					i++;
				}
				else
				{
					System.Diagnostics.Debug.Assert(false);
					//m_Properties.SelectedObject = Thing; 
					//return;
				}
			}

			NewSet[NewSet.Length - 1] = Thing;
			m_Properties.SelectedObjects = NewSet; 
		}

		/// <summary>Remove um elemento do array SelectedObjects do Property Grid</summary>
		private void RemoveFromPropertyGrid (DocElement Thing)
		{
			if (m_Properties == null)
				return;

			if (m_Properties.SelectedObjects.Length > 1)
			{
				object [] NewSet = new object[m_Properties.SelectedObjects.Length - 1];
				int i = 0;

				foreach (DocElement EachThing in m_Properties.SelectedObjects)
					if ((EachThing != Thing) && (EachThing != null))
					{
						NewSet[i] = EachThing;
						i++;
					}

				try
				{
					m_Properties.SelectedObjects = NewSet; 
				}
				catch (System.Exception e)
				{
					System.Console.Out.Write (e.ToString ());
					m_Properties.SelectedObjects = null;
				}
			}
			else if (m_Properties.SelectedObject == Thing)
				m_Properties.SelectedObject = null;
		}

	}
}
